package extensions;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.net.Socket;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.Set;
import java.util.Map;
import java.util.HashMap;
import java.util.ArrayList;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import dareka.extensions.Extension2;
import dareka.extensions.ExtensionManager;
import dareka.processor.HttpRequestHeader;
import dareka.processor.StringResource;
import dareka.processor.Resource;
import dareka.processor.Processor;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.Statement;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.ResultSetMetaData;

public class UnlimitedMyList implements Extension2, Processor {
	public static final String UMYLIST_VERSION = "Unlimited MyList 1.8";
	public static final String LOG_PREFIX = "umylist";
	public static final String PROP_DEBUG = "uMyListDebug";
	private static final String[] PROCESSOR_SUPPORTED_METHODS = new String[] { "GET" };
	private static final Pattern PROCESSOR_SUPPORTED_PATTERN = Pattern.compile(
			"^(http://[^/]+\\.(?:nico|smile)video\\.jp)/umylist(/.*)$");
	private static ExtUtil util;
	private static Connection con;
	private static Statement stmt;
	
	private static final String STATUS = "status";
	
	public HashMap<String,String> hash_;
	
	public UnlimitedMyList(){
		hash_ = new HashMap<>();
	}
	
	/*
	内部クラスを使わないように
	パフォーマンスはそこまで悪くなかったため採用
	*/
	// 要素の追加
	UnlimitedMyList put(String key,String value) {
		hash_.put(key,value);
		return this;
	}
	// 取得
	String get(String key) { return (String)hash_.get(key); }
	UnlimitedMyList gen(){ return new UnlimitedMyList(); }
	
	/*
	バージョンアップの際に列を追加したい時に
	*/
	// テーブルにtype型のcolNameを追加
	private void updateTable(String tableName,String colName,String type) {
		boolean e = false;
		final String pre = "updateTable()";
		try {
			ResultSet rs = stmt.executeQuery("PRAGMA table_info('"+tableName+"')");
			while( rs.next() ) {
				if( rs.getString("name").equals(colName) ) {
					e = true;
					break;
				}
			}
			rs.close();
			if( !e ) {
				debug(pre,"ALTER TABLE "+tableName+" ADD COLUMN "+colName+" "+type);
				stmt.execute("ALTER TABLE "+tableName+" ADD COLUMN "+colName+" "+type);
			}
		} catch( Exception exc ) {
		}
		debug(pre,colName+" "+e);
	}
	
	// テーブルにデフォルト値defでtype型のcolNameを追加
	private void updateTable(String tableName,String colName,String type,String def) {
		boolean e = false;
		final String pre = "updateTable()";
		try {
			debug(pre,"PRAGMA table_info('"+tableName+"')");
			ResultSet rs = stmt.executeQuery("PRAGMA table_info('"+tableName+"')");
			while( rs.next() ) {
				debug(pre,rs.getString("name"));
				if( rs.getString("name").equals(colName) ) {
					e = true;
				}
			}
			rs.close();
			if( !e ) {
				info(pre,"ALTER TABLE "+tableName+" ADD COLUMN "+colName+" "+type);
				stmt.execute("ALTER TABLE "+tableName+" ADD COLUMN "+colName+" "+type+" default '"+def+"'");
			}
		} catch( Exception exc ) {
			debug(pre,"err:"+exc);
		}
		debug(pre,colName+" "+e);
	}

	// extensionの登録
	public void registerExtensions(ExtensionManager mgr) {
		final String pre = "registerExtensions()";
		if( util == null ) {
			util = new ExtUtil(this, LOG_PREFIX, PROP_DEBUG);
		}
		/* クラスの読み込みよくわからないので誤魔化した */
		try {
			URL[] urls = new URL[1];
			
			urls[0] = new URL("jar:file:/sqlite-jdbc.jar!/");
			
			ClassLoader url = new URLClassLoader(urls);
			Class.forName("org.sqlite.JDBC");
		} catch(Exception e) {
		}
		if( con == null ) {
			try {
				con = DriverManager.getConnection("jdbc:sqlite:umylist.db");	
				stmt = con.createStatement();
				// if not existsがあったのでこっち使う
				stmt.execute("create table if not exists umylist(id INTEGER PRIMARY KEY AUTOINCREMENT,name text)");
				stmt.execute("create table if not exists umovie(id INTEGER PRIMARY KEY AUTOINCREMENT,mylistid integer,movieid text)");
				stmt.execute("create table if not exists mylist(id INTEGER PRIMARY KEY AUTOINCREMENT,mylistid integer)");
				updateTable("umovie","threadid","text","-1");
				updateTable("mylist","name","text","");
			} catch(Exception e) {
				info(pre,e+"");
			}
		}
		mgr.registerProcessor(this);      // Processorとして登録
	}
	
	public String getVersionString() {
		return UMYLIST_VERSION;
	}

	public String[] getSupportedMethods() {
		return PROCESSOR_SUPPORTED_METHODS;
	}

	public Pattern getSupportedURLAsPattern() {
		return PROCESSOR_SUPPORTED_PATTERN;
	}
	
	public String getSupportedURLAsString() {
		return null;
	}
	
	// URLのクエリーを取得
	private HashMap getQueryParameter(String s) {
		HashMap<String,String> map = new HashMap<String,String>();
		String[] v = s.split("&");
		for(int i = 0; i < v.length; i++) {
			String[] w = v[i].split("=");
			if( w.length == 2 ) {
				map.put(w[0],w[1]);
			} else {
				map.put(w[0],"");
			}
		}
		return map;
	}
	
	// XSS対策周りのエスケープ
	private String replace(String str) {
		String ret = new String(str);
		String[] escape = {"&", "<", ">", "\"", "\'"};
		String[] replace = {"&amp;", "&lt;", "&gt;", "&quot;", "&#39;"};
		for ( int i=0; i < escape.length; i++ ){
			ret = ret.replace(escape[i], replace[i]);
		}
		return ret;
	}
	
	// 許される動画ID？
	private boolean isAcceptMID(String id) {
		Pattern pat = Pattern.compile("^[a-z]{2}\\d+$");
		Matcher mt = pat.matcher(id);
		return mt.matches();
	}
	
	// 許されるスレッドID？
	private boolean isAcceptTID(String id) {
		Pattern pat = Pattern.compile("^\\d+$");
		Matcher mt = pat.matcher(id);
		return mt.matches();
	}
	
	// 許されるマイリストID？
	private boolean isAcceptMylistID(String id) {
		Pattern pat = Pattern.compile("^\\d+$");
		Matcher mt = pat.matcher(id);
		return mt.matches();
	}
	
	// 指定したUマイリストの動画の数
	private int getMovieCount(int id) {
		return getMovieCount(id+"");
	}

	// 指定したUマイリストの動画の数
	private int getMovieCount(String id) {
		try {
			PreparedStatement pstmt = con.prepareStatement("SELECT COUNT(*) FROM umovie where mylistid=?");
			pstmt.setString(1,id);
			ResultSet rs = pstmt.executeQuery();
			rs.next();
			int ret = rs.getInt(1);
			rs.close();
			return ret;
		} catch(Exception e) {
			return 0;
		}
	}

	private String getMovies(int id) {
		return getMovies(id+"");
	}
	
	private String getMovies(String id) {
		try {
			String ret = "";
			String sql = "select * from umovie where mylistid=?";
			PreparedStatement pstmt = con.prepareStatement(sql);
			pstmt.setString(1,replace(id));
			ResultSet rs = pstmt.executeQuery();
			while( rs.next() ) {
				if( !ret.equals("") ) ret += ",";
				ret += "{\"movieid\":\""+rs.getString("movieid")+"\",\"threadid\":\""+rs.getString("threadid")+"\"}";
			}
			return ret;
		} catch(Exception e) {
			return "";
		}
	}
	
	private void info(String path,String message) {
		util.info(path+" ["+message+"]");
	}
	
	private void debug(String path,String message) {
		if( System.getProperty("umylistDebug","false").equals("true") ) {
			info(path,"debug:"+message);
		}
	}
	
	private ArrayList<UnlimitedMyList> getuMyList() {
		final String pre = "getMyList()";
		ArrayList<UnlimitedMyList> ret = new ArrayList<>();
		try {
			ResultSet rs = stmt.executeQuery("select * from umylist");
			ResultSetMetaData rsmd = rs.getMetaData();
			for(int now = 0; rs.next(); now++) {
				String id = rs.getString("id");
				String name = rs.getString("name");
				ret.add(gen().put("id",id).put("name",name));
			}
			debug("getuMyList()","num="+ret.size());
			rs.close();
			return ret;
		} catch(SQLException sqle) {
			debug(pre,sqle+"");
		}
		return ret;
	}
	
	private ArrayList<UnlimitedMyList> getMyList() {
		final String pre = "getMyList()";
		ArrayList<UnlimitedMyList> ret = new ArrayList<>();
		try {
			ResultSet rs = stmt.executeQuery("select * from mylist");
			while( rs.next() ) {
				String id = rs.getString("id");
				String myListId = rs.getString("mylistid");
				String name = rs.getString("name");
				ret.add(gen().put("id",id).put("mylistid",myListId).put("name",name));
			}
			rs.close();
		} catch(SQLException sqle) {
			debug(pre,sqle+"");
		}
		return ret;
	}
	
	private ArrayList<UnlimitedMyList> getuMyList(int uMyListId) {
		return getuMyList(uMyListId+"");
	}
	
	private ArrayList<UnlimitedMyList> getuMyList(String uMyListId) {
		final String pre = "getuMovie()";
		ArrayList<UnlimitedMyList> ret = new ArrayList<>();
		String sql = "select * from umovie where mylistid=?";
		try {
			PreparedStatement pstmt = con.prepareStatement(sql);
			pstmt.setString(1,uMyListId);
			ResultSet rs = pstmt.executeQuery();
			while( rs.next() ) {
				String movieId = rs.getString("movieid");
				String threadId = rs.getString("threadid");
				ret.add(new UnlimitedMyList().put("movieid",movieId).put("threadid",threadId));
			}
			rs.close();
		} catch(SQLException sqle) {
			debug(pre,sqle+"");
		}
		return ret;
	}
	
	private String encodeName(String name) {
		final String pre = "encodeName()";
		try {
			return (String)replace(URLDecoder.decode((String)name,"utf8"));
		} catch(UnsupportedEncodingException uee) {
			debug(pre,uee+"");
		}
		return "";
	}
	
	private boolean isExistName(String name) {
		String pre = "isExistName()";
		String sql = "SELECT COUNT(*) FROM umylist where name=?";
		try {
			String ename = encodeName(name);
			PreparedStatement pstmt = con.prepareStatement(sql);
			pstmt.setString(1,ename);
			ResultSet rs = pstmt.executeQuery();
			rs.next();
			int num = rs.getInt(1);
			rs.close();
			debug(pre,"num="+num);
			return num!=0;
		} catch(SQLException sqle) {
			debug(pre,sqle+"");
		}
		return false;
	}
	
	private boolean isValiduMyListName(String name) {
		return !name.equals("");
	}
	
	private boolean isValidMyListName(String name) {
		return !name.equals("");
	}
	
	private UnlimitedMyList adduMyList(String name) {
		final String pre = "adduMyList()";
		String sql = "INSERT INTO umylist (name) VALUES(?)";
		String ename = encodeName(name);
		if( !isValiduMyListName(ename) ) { return null; }
		try {
			PreparedStatement pstmt = con.prepareStatement(sql,PreparedStatement.RETURN_GENERATED_KEYS);
			pstmt.setString(1,ename);
			pstmt.execute();
			ResultSet rs = pstmt.getGeneratedKeys();
			rs.next();
			String id = rs.getString(1);
			debug(pre,"id="+id);
			return gen().put("id",id).put("name",ename);
		} catch(SQLException sqle) {
			debug(pre,sqle+"");
		}
		return null;
	}

	private boolean isExistuMyList(int uMyListId) {
		return isExistuMyList(uMyListId+"");
	}
	
	private boolean isExistuMyList(String uMyListId) {
		final String pre = "isExistuMyList()";
		String sql = "SELECT count(*) FROM umylist WHERE id=?";
		try {
			PreparedStatement pstmt = con.prepareStatement(sql);
			pstmt.setString(1,uMyListId);
			ResultSet rs = pstmt.executeQuery();
			rs.next();
			int num = rs.getInt(1);
			rs.close();
			debug(pre,"num="+num);
			return num!=0;
		} catch(SQLException sqle) {
			debug(pre,sqle+"");
		}
		return false;
	}
	
	private boolean isExistMyList(int myListId) {
		return isExistMyList(myListId);
	}
	
	private boolean isExistMyList(String myListId) {
		String pre = "isExistMyList()";
		String sql = "SELECT COUNT(*) FROM mylist where mylistid=?";
		try {
			PreparedStatement pstmt = con.prepareStatement(sql);
			pstmt.setString(1,myListId);
			ResultSet rs = pstmt.executeQuery();
			rs.next();
			int num = rs.getInt(1);
			rs.close();
			debug(pre,"num="+num);
			return num != 0;
		} catch(SQLException sqle) {
			debug(pre,sqle+"");
		}
		return false;
	}
	
	private UnlimitedMyList addMyList(String myListId,String name) {
		String pre = "addMyList()";
		String sql = "INSERT INTO mylist (mylistid,name) VALUES(?,?)";
		String ename = encodeName(name);
		debug(pre,"myListId="+myListId+" ename="+ename);
		if(!isValidMyListName(ename))return null;
		try {
			PreparedStatement pstmt = con.prepareStatement(sql,PreparedStatement.RETURN_GENERATED_KEYS);
			pstmt.setString(1,myListId);
			pstmt.setString(2,ename);
			pstmt.execute();
			ResultSet rs = pstmt.getGeneratedKeys();
			rs.next();
			String id = rs.getString(1);
			debug(pre,"id="+id);
			return gen().put("id",id).put("name",ename).put("mylistid",myListId);
		} catch(SQLException sqle) {
			debug(pre,sqle+"");
		}
		return null;
	}
	
	private boolean isExistuMovie(String uMyListId,String movieId,String threadId) {
		final String pre = "isExistuMovie()";
		String sql = "SELECT count(*) FROM umovie WHERE mylistid=? AND movieid=? AND threadid=?";
		try {
			PreparedStatement pstmt = con.prepareStatement(sql);
			pstmt.setString(1,uMyListId);
			pstmt.setString(2,movieId);
			pstmt.setString(3,threadId);
			ResultSet rs = pstmt.executeQuery();
			rs.next();
			int num = rs.getInt(1);
			rs.close();
			debug(pre,"num="+num);
			return num!=0;
		} catch(SQLException sqle) {
			debug(pre,sqle+"");
		}
		return false;
	}
	
	private UnlimitedMyList adduMovie(String uMyListId,String movieId,String threadId) {
		final String pre = "adduMovie()";
		String sql = "INSERT INTO umovie (mylistid,movieid,threadid) VALUES(?,?,?)";
		if( isExistuMovie(uMyListId,movieId,threadId) ) {
			return null;
		}
		try {
			PreparedStatement pstmt = con.prepareStatement(sql);
			pstmt.setString(1,uMyListId);
			pstmt.setString(2,movieId);
			pstmt.setString(3,threadId);
			pstmt.execute();
			return gen().put("mylistid",uMyListId).put("movieid",movieId).put("threadid",threadId);
		} catch(SQLException sqle) {
			debug(pre,sqle+"");
		}
		return null;
	}
	
	private boolean removeuMovie(String uMyListId,String movieId,String threadId) {
		final String pre = "removeuMovie()";
		String sql = "DELETE FROM umovie WHERE mylistid=? AND movieid=? AND threadid=?";
		try {
			PreparedStatement pstmt = con.prepareStatement(sql);
			pstmt.setString(1,uMyListId);
			pstmt.setString(2,movieId);
			pstmt.setString(3,threadId);
			pstmt.execute();
			return true;
		} catch(SQLException sqle) {
			debug(pre,sqle+"");
		}
		return false;
	}
	
	private boolean removeuMovie(String uMyListId) {
		final String pre = "removeuMovie()";
		String sql = "DELETE FROM umovie WHERE mylistid=?";
		try {
			PreparedStatement pstmt = con.prepareStatement(sql);
			pstmt.setString(1,uMyListId);
			pstmt.execute();
			return true;
		} catch(SQLException sqle) {
			debug(pre,sqle+"");
		}
		return false;
	}
	
	private boolean removeuMyList(String uMyListId) {
		final String pre = "removeuMyList()";
		String sql = "DELETE FROM umylist WHERE id=?";
		try {
			PreparedStatement pstmt = con.prepareStatement(sql);
			pstmt.setString(1,uMyListId);
			pstmt.execute();
			return true;
		} catch(SQLException sqle) {
			debug(pre,sqle+"");
		}
		return false;
	}
	
	private UnlimitedMyList renameuMyList(String uMyListId,String name) {
		final String pre = "renameuMyList()";
		String sql = "UPDATE umylist SET name=? WHERE id=?";
		String ename = encodeName(name);
		if( !isValiduMyListName(ename) ) {
			return null;
		}
		try {
			PreparedStatement pstmt = con.prepareStatement(sql);
			pstmt.setString(1,ename);
			pstmt.setString(2,uMyListId);
			pstmt.execute();
			return gen().put("mylistid",uMyListId).put("name",ename);
		} catch(SQLException sqle) {
			debug(pre,sqle+"");
		}
		return null;
	}
	
	private UnlimitedMyList renameMyList(String myListId,String name) {
		final String pre = "renameMyList()";
		String sql = "UPDATE mylist SET name=? WHERE mylistid=?";
		String ename = encodeName(name);
		if( !isValiduMyListName(ename) ) {
			return null;
		}
		try {
			PreparedStatement pstmt = con.prepareStatement(sql);
			pstmt.setString(1,ename);
			pstmt.setString(2,myListId);
			pstmt.execute();
			return gen().put("mylistid",myListId).put("name",ename);
		} catch(SQLException sqle) {
			debug(pre,sqle+"");
		}
		return null;
	}
	
	private boolean removeMyListById(int id) {
		return removeMyListById(id+"");
	}
	
	private boolean removeMyListById(String id) {
		final String pre = "removeMyListById()";
		String sql = "DELETE FROM mylist WHERE id=?";
		try {
			PreparedStatement pstmt = con.prepareStatement(sql);
			pstmt.setString(1,id);
			pstmt.execute();
			return true;
		} catch(SQLException sqle) {
			debug(pre,sqle+"");
		}
		return false;
	}

	private boolean removeMyList(int myListId) {
		return removeMyList(myListId+"");
	}
	
	private boolean removeMyList(String myListId) {
		final String pre = "removeMyList()";
		String sql = "DELETE FROM mylist WHERE mylistid=?";
		try {
			PreparedStatement pstmt = con.prepareStatement(sql);
			pstmt.setString(1,myListId);
			pstmt.execute();
			return true;
		} catch(SQLException sqle) {
			debug(pre,sqle+"");
		}
		return false;
	}
	
	// ArrayList<T>とか使えない.String,String
	String getHashAsJson() {
		StringBuilder ret = new StringBuilder();
		Set<Map.Entry<String,String>> set = hash_.entrySet();
		boolean flg = false;
		ret.append("{");
		for(Map.Entry<String,String> k: set) {
			if( flg ) ret.append(",");
			else flg = true;
			ret.append("\""+k.getKey()+"\":\""+k.getValue()+"\"");
		}
		ret.append("}");
		return ret.toString();
	}
	
	UnlimitedMyList makeResponse(String status) {
		return gen().put(STATUS,status);
	}
	
	// /umylist/madd
	UnlimitedMyList makeResponseuMadd(String status,String id,String name) {
		return makeResponse(status).put("mylist",id).put("name",name);
	}
	
	// /umylist/add
	UnlimitedMyList makeResponseuAdd(String status,String myListId,String movieId,String threadId) {
		return makeResponse(status).put("mylist",myListId).put("movieid",movieId).put("threadid",threadId);
	}
	
	// /umylist/rename
	UnlimitedMyList makeResponseuRename(String status,String myListId,String name) {
		return makeResponseuMadd(status,myListId,name);
	}
	
	// /umylist/mylist/add
	UnlimitedMyList makeResponseAdd(String status,String myListId,String name) {
		return makeResponse(status).put("mylist",myListId).put("name",name);
	}
	
	// /umylist/mylist/rename
	UnlimitedMyList makeResponseRename(String status,String myListId,String name) {
		return makeResponse(status).put("mylist",myListId).put("name",name);
	}
	
	UnlimitedMyList makeResponseException(String status,String e) {
		return makeResponse(status).put("exception",e);
	}
	
	StringResource responseJson(String str) {
		return util.responseText(str,"application/json; charset=UTF-8");
	}
	
	public Resource onRequest(HttpRequestHeader requestHeader, Socket browser)
	throws IOException {
		try {
			Matcher m = PROCESSOR_SUPPORTED_PATTERN.matcher(requestHeader.getURI());
			if (!m.matches()) {
				throw new IOException("");
			}
			if( m.groupCount() < 2 ) {
				return responseJson(makeResponse("err").getHashAsJson());
//				return util.responseText("err", "text/plain; charset=UTF-8"); 
			}
			String a = m.group(1), b = m.group(2);
			String[] tmp = b.split("\\?");
			String path = tmp[0];
			HashMap get = new HashMap();
			if( tmp.length != 1 ) {
				get = getQueryParameter(tmp[1]);
			}
			StringBuilder res = new StringBuilder();
			if( path.equals("/list") ) {
				res.append("<?xml version=\"1.0\" encoding=\"utf-8\"?>\n");
				if( get.get("v") == null ) {
					ArrayList<UnlimitedMyList> umls = getuMyList();
					res.append("<mylist>\n");
					for(UnlimitedMyList uml : umls) {
						res.append("<list>\n");
						res.append("<id>"+uml.get("id")+"</id>\n");
						res.append("<name>"+uml.get("name")+"</name>\n");
						res.append("<moviecount>"+getMovieCount(uml.get("id"))+"</moviecount>\n");
						res.append("</list>\n");
					}
					res.append("</mylist>\n");
				} else {
					String mid = (String)get.get("v");
					ArrayList<UnlimitedMyList> ums = getuMyList(mid);
					debug(path,"v="+mid);
					res.append("<mylist>\n");
					res.append("<moviecount>"+ums.size()+"</moviecount>\n");
					for(UnlimitedMyList um : ums) {
						res.append("<list>\n");
						res.append("<id>"+um.get("movieid")+"</id>\n");
						res.append("<threadid>"+um.get("threadid")+"</threadid>\n");
						res.append("</list>\n");
					}
					res.append("</mylist>\n");
				}
				return util.responseText(res.toString(),"application/xml; charset=UTF-8");
//				return new StringResource(res.toString());
			} else if( path.equals("/madd") ) {
				if( get.get("name") == null ) {
					return responseJson(makeResponse("failed").getHashAsJson());
//					return util.responseText("failed", "text/plain; charset=UTF-8");
				}
				String name = (String)get.get("name");
				debug(path,"name="+name);
				// 設定されていて同じ名前のマイリストがすでに入っている
				if( !System.getProperty("umylistUniqueMyList", "false").equals("false") && isExistName(name)) {
					debug(path,"overlap");
					return responseJson(makeResponse("overlap").getHashAsJson());
//					return util.responseText("overlap", "text/plain; charset=UTF-8");
				}
				if( !isValiduMyListName(name) ) {
					return responseJson(makeResponse("invalidName").getHashAsJson());
//					return util.responseText("invalidName", "text/plain; charset=UTF-8"); 
				}
				UnlimitedMyList um = adduMyList(name);
				if( um == null ) {
					return responseJson(makeResponse("failed").getHashAsJson());
//					return util.responseText("failed", "text/plain; charset=UTF-8"); 
				}
				String id = um.get("id");
				return responseJson(makeResponseuMadd("succeeded",id,name).getHashAsJson());
//				return util.responseText("succeeded", "text/plain; charset=UTF-8"); 
			} else if( path.equals("/add") ) {
				if( get.get("mid") != null && get.get("mylist") != null && get.get("tid") != null ) {
					String mylist = replace((String)get.get("mylist"));
					String mid = replace((String)get.get("mid"));
					String tid = replace((String)get.get("tid"));
					debug(path,"mylist="+mylist+" mid="+mid+" tid"+tid);
					if( !isAcceptMID(mid) || !isAcceptTID(tid) ) {
						return responseJson(makeResponse("invalidID").getHashAsJson());
//						return util.responseText("invalidID", "text/plain; charset=UTF-8"); 
					}
					if( isExistuMovie(mylist,mid,tid) ) {
						return responseJson(makeResponse("overlap").getHashAsJson());
//						return util.responseText("overlap", "text/plain; charset=UTF-8"); 
					}
					if( !isExistuMyList(mylist) ) {
						return responseJson(makeResponse("invalidMylist").getHashAsJson());
//						return util.responseText("invalidMylist", "text/plain; charset=UTF-8");
					}
					UnlimitedMyList um = adduMovie(mylist,mid,tid);
					if( um == null ) {
						return responseJson(makeResponse("failed").getHashAsJson());
//						return util.responseText("failed", "text/plain; charset=UTF-8");
					}
					String myListId = um.get("mylistid");
					String movieId = um.get("movieid");
					String threadId = um.get("threadid");
					return responseJson(makeResponseuAdd("succeeded",myListId,movieId,threadId).getHashAsJson());
//					return util.responseText("succeeded", "text/plain; charset=UTF-8");
				} else {
					return responseJson(makeResponse("failed").getHashAsJson());
//					return util.responseText("failed", "text/plain; charset=UTF-8"); 
				}
			} else if( path.equals("/remove") ) {
				if( get.get("mid") != null && get.get("mylist") != null  && get.get("tid") != null ) {
					String mylist = (String)get.get("mylist");
					String mid = (String)get.get("mid");
					String tid = (String)get.get("tid");
					debug(path,"mylist="+mylist+" mid="+mid+" tid"+tid);
					if( !removeuMovie(mylist,mid,tid) ) {
						return responseJson(makeResponse("failed").getHashAsJson());
//						return util.responseText("failed", "text/plain; charset=UTF-8");
					}
					return responseJson(makeResponse("succeeded").getHashAsJson());
//					return util.responseText("succeeded", "text/plain; charset=UTF-8");
				} else {
					return responseJson(makeResponse("failed").getHashAsJson());
//					return util.responseText("failed", "text/plain; charset=UTF-8");
				}
			} else if( path.equals("/mremove") ) {
				if( get.get("mylist") != null ) {
					String mylist = (String)get.get("mylist");
					debug(path,"mylist="+mylist);
					if( !removeuMyList(mylist) || !removeuMovie(mylist) ) {
						return responseJson(makeResponse("failed").getHashAsJson());
//						return util.responseText("failed", "text/plain; charset=UTF-8"); 
					}
					return responseJson(makeResponse("succeeded").getHashAsJson());
//					return util.responseText("succeeded", "text/plain; charset=UTF-8"); 
				} else {
					return responseJson(makeResponse("failed").getHashAsJson());
//					return util.responseText("failed", "text/plain; charset=UTF-8"); 
				}
			} else if( path.equals("/ajax") ) {
				ArrayList<UnlimitedMyList> umls = getuMyList();
				for(int i = 0; i < 2; i++) {
					StringBuilder sb = new StringBuilder();
					String s = (i==0?"{":"["),e = (i==0?"}":"]");
					sb.append(s+"\n");
					boolean flg = false;
					for(UnlimitedMyList uml : umls) {
						if( flg ) {
							sb.append(",\n");
						} else {
							flg = true;
						}
						String uMylistId = uml.get("id");
						String name = uml.get("name");
						sb.append((i==0?"\""+uMylistId+"\":{":"{\"id\":"+uMylistId+",")+"\"name\":\""+name+"\",\"movies\":["+getMovies(uMylistId)+"]}");
					}
					sb.append("\n"+e+";");
					res.append("uMylist"+i+" = ");
					res.append(sb);
					res.append("\n");
				}
				return util.responseText(res.toString(),"text/javascript; charset=UTF-8");
			} else if( path.equals("/rename") ) {
				if( get.get("mylist") != null && get.get("name") != null ) {
					String mylist = replace((String)get.get("mylist"));
					String name = (String)get.get("name");
					debug(path,"mylist="+mylist+" name="+name);
					if( !isValiduMyListName(name) ) {
						return responseJson(makeResponse("invalidName").getHashAsJson());
//						return util.responseText("invalidName", "text/plain; charset=UTF-8"); 
					}
					// 設定されていて同じ名前のマイリストがすでに入っている
					if( !System.getProperty("umylistUniqueMyList", "false").equals("false") && isExistName(name) ) {
						debug(path,"overlap");
						return responseJson(makeResponse("overlap").getHashAsJson());
//						return util.responseText("overlap", "text/plain; charset=UTF-8"); 
					}
					UnlimitedMyList um = renameuMyList(mylist,name);
					if( um == null ) {
						return responseJson(makeResponse("failed").getHashAsJson());
//						return util.responseText("failed","text/javascript; charset=UTF-8");
					}
					mylist = um.get("mylistid");
					name = um.get("name");
					return responseJson(makeResponseuRename("succeeded",mylist,name).getHashAsJson());
//					return util.responseText("succeeded","text/javascript; charset=UTF-8");
				} else {
					return responseJson(makeResponse("failed").getHashAsJson());
//					return util.responseText("failed", "text/plain; charset=UTF-8"); 
				}
			} else if( path.equals("/mylist/add") ) {
				if( get.get("mylist") != null && get.get("name") != null ) {
					String myListId = (String)get.get("mylist");
					String name = (String)get.get("name");
					if( !isAcceptMylistID(myListId) ) {
						return responseJson(makeResponse("invalidMylist").getHashAsJson());
//						return util.responseText("invalidMylist", "text/plain; charset=UTF-8");
					}
					if( isExistMyList(myListId) ){
						return responseJson(makeResponse("overlap").getHashAsJson());
//						return util.responseText("overlap", "text/plain; charset=UTF-8");
					}
					UnlimitedMyList um = addMyList(myListId,name);
					if( um == null ) {
						return responseJson(makeResponse("failed").getHashAsJson());
//						return util.responseText("failed", "text/plain; charset=UTF-8"); 
					}
					String id = um.get("id");
					myListId = um.get("mylistid");
					name = um.get("name");
					return responseJson(makeResponseAdd("succeeded",myListId,name).getHashAsJson());
//					return util.responseText("succeeded", "text/plain; charset=UTF-8"); 
				} else {
					return responseJson(makeResponse("failed").getHashAsJson());
//					return util.responseText("failed", "text/plain; charset=UTF-8"); 
				}
			} else if( path.equals("/mylist/remove") ) {
				if( get.get("id") != null ) {
					String id = (String)get.get("id");
					debug(path,"id="+id);
					if( !removeMyListById(id) ) {
						return responseJson(makeResponse("failed").getHashAsJson());
//						return util.responseText("failed", "text/plain; charset=UTF-8"); 
					}
					return responseJson(makeResponse("succeeded").getHashAsJson());
//					return util.responseText("succeeded", "text/plain; charset=UTF-8"); 
				} else if( get.get("mylist") != null ) {
					String myListId = (String)get.get("mylist");
					debug(path,"myListId="+myListId);
					if( !removeMyList(myListId) ) {
						return responseJson(makeResponse("failed").getHashAsJson());
//						return util.responseText("failed", "text/plain; charset=UTF-8"); 
					}
					return responseJson(makeResponse("succeeded").getHashAsJson());
				} else {
					return responseJson(makeResponse("failed").getHashAsJson());
				}
			} else if( path.equals("/mylist/list") ) {
				ArrayList<UnlimitedMyList> mls = getMyList();
				res.append("<?xml version=\"1.0\" encoding=\"utf-8\"?>\n");
				res.append("<mylist>\n");
				for(UnlimitedMyList ml : mls) {
					res.append("<list>\n");
					res.append("<id>"+ml.get("id")+"</id>\n");
					res.append("<mylistid>"+ml.get("mylistid")+"</mylistid>\n");
					res.append("<name>"+ml.get("name")+"</name>\n");
					res.append("</list>\n");
				}
				res.append("</mylist>\n");
				return util.responseText(res.toString(),"application/xml; charset=UTF-8");
			} else if( path.equals("/mylist/ajax") ) {
				ArrayList<UnlimitedMyList> mls = getMyList();
				for(int i = 0; i < 2; i++) {
					StringBuilder sb = new StringBuilder();
					String s = (i==0?"{":"["),e = (i==0?"}":"]");
					sb.append(s+"\n");
					boolean flg = false;
					for(UnlimitedMyList ml : mls) {
						if( flg ) {
							sb.append(",\n");
						} else {
							flg = true;
						}
						String mMylistId = ml.get("id");
						String mylistId = ml.get("mylistid");
						String name = ml.get("name");
						sb.append((i==0?("\""+mMylistId+"\":{"):("{\"id\":"+mMylistId+","))+"\"mylistid\":\""+mylistId+"\",\"name\":\""+name+"\"}");
					}
					sb.append("\n"+e+";");
					res.append("umMylist"+i+" = ");
					res.append(sb);
					res.append("\n");
				}
				return util.responseText(res.toString(),"text/javascript; charset=UTF-8");
			} else if( path.equals("/mylist/rename") ) {
				if( get.get("mylist") != null && get.get("name") != null ) {
					String mylist = replace((String)get.get("mylist"));
					String name = (String)get.get("name");
					debug(path,"mylist="+mylist+" name="+name);
					if( !isValidMyListName(name) ) {
						return responseJson(makeResponse("invalidName").getHashAsJson());
					}
					UnlimitedMyList um = renameMyList(mylist,name);
					if( um == null ) {
						return responseJson(makeResponse("failed").getHashAsJson());
					}
					mylist = um.get("mylistid");
					name = um.get("name");
					return responseJson(makeResponseRename("succeeded",mylist,name).getHashAsJson());
				} else {
					return responseJson(makeResponse("failed").getHashAsJson());
				}
			}
		} catch(Exception e) {
			return responseJson(makeResponseException("failed",e+"").getHashAsJson());
//			return util.responseText(e+"", "text/plain; charset=UTF-8"); 
		}
		return responseJson(gen().getHashAsJson());
//		return util.responseText("", "text/plain; charset=UTF-8"); 
	}
}